local type, tostring, getmetatable, unpack = type, tostring, getmetatable, unpack
local format, debug = string.format, debug
local popen = io.popen

module('nsg')

local function basic_subst(s,t)
  return (s:gsub('%$([%w_]+)',t))
end


local function formatx (fmt,...)
  local args = {...}
  local i = 1
  for p in fmt:gmatch('%%.') do
    if p == '%s' and type(args[i]) ~= 'string' then
      args[i] = tostring(args[i])
    end
    i = i + 1
  end
  return format(fmt,unpack(args))
end

function meta()
  local strmeta = getmetatable("")
  strmeta.__mod = function(a, b)
    if b == nil then
      return a
    elseif type(b) == "table" and getmetatable(b) == nil then
      if #b == 0 then -- assume a map-like table
        return basic_subst(a,b)
      else
        return formatx(a,unpack(b))
      end
    elseif type(b) == 'function' then
      return basic_subst(a,b)
    else
      return formatx(a,b)
    end
  end

  strmeta.__unm = function(str)
    local f, err = popen(str)
    if f then
      local s
      s, err = f:read("*a")
      f:close()
      f = s
    end
    return f, err
  end

  debug.setmetatable(type,{
      __concat = function(f1,f2)
        return function(...) return f1(f2(...)) end
      end;
  })
end
